function [ beta_hat,p ] = semi_sup_logistic_dropout_approx( X, y, delta, Z, alpha, epsilon,max_it )
%%
% function [ beta_hat,p ] = semi_sup_logistic_dropout_approx( X, y, delta, 
%                                                Z, alpha, epsilon,max_it )

[n,d] = size(X);
[m,d2] = size(Z);
if d~=d2
  disp('X and Z should have same second dim.');
end

get_p = @(X,beta) 1./(1+exp(-X*beta));

% try alternating minimzation over f = -l(beta) + Rq*(beta,p)
beta = zeros(d,1);
p = get_p(X,beta);
q = get_p(Z,beta);
lambda = delta/(1-delta) * n/(n+alpha*m);
converged = 0;
it = 1;
%figure; hold all;
while ~converged
  % optimize beta with weighted elastic-net (alpha = 0 --> l2 reg.),
  % with fixed p (from previous beta)
  pf = (p.*(1-p))'*X.^2 + alpha*(q.*(1-q))'*Z.^2;
  options.alpha = 0;
  options.lambda = lambda;
  options.penalty_factor = pf;
  beta_hat = glmnet(X,y,'binomial',options);
  beta_hat = beta_hat.beta;
  
  % optimize p, with fixed beta = beta_hat
  p_hat = get_p(X,beta_hat);
  q_hat = get_p(Z,beta_hat);
  
  if norm([p;q]-[p_hat;q_hat]) < epsilon || it == max_it
    converged = 1;
  end
  
  pq_norms(it) = norm([p;q]-[p_hat;q_hat]);
  
  beta = beta_hat;
  p = p_hat;
  q = q_hat;
  it = it+1;
  
  %  plot(pq_norms); drawnow
  
end

end