% simulations.m
% script to run simulations for stats jc presentation
% Dropout Training as Adaptive Regularization
%    Wager et al., 2013


% Simulation described in A.1 (p.10)

predict_y = @(X,beta_hat) 0.5 < 1./(1+exp(-X*beta_hat));
d = 1050;
d_sig = 50;
n_train = 75;
n_test = 75;

num_copiess = [1 3 5 10 20 50 75 100];   
deltas = [0.1 0.3 0.5 0.7 0.9];% 0.95];
add_sigmas = [0.01 0.1 0.5 1 5];% 10];
assert(numel(deltas) == numel(add_sigmas));

%% generating data
num_trials = 100;
%wbar = waitbar(0,'Multiple Trials...');
for n = 1:num_trials

  fprintf('Trial %d\n',n);
  
    [ X_train, y_train, beta ] = gen_data( d, d_sig, n_train );

    [ X_test, y_test ] = gen_data( d, d_sig, n_test, beta );

    %% l2 lambda cross validation

    % n_val = 500;
    % [ X_val, y_val ] = gen_data( d, d_sig, n_val, beta );
    % lambdas = [0 0.01 0.1 1 5 10 20 25 30 35 40 50];
    % beta_hat_l2s = logistic_l2(X_train,y_train,lambdas);
    % y_val_hat_l2s = zeros(size(y_val,1),numel(lambdas));
    % acc_l2_all = zeros(numel(lambdas),1);
    % acc_l2_active = zeros(numel(lambdas),1);
    % for l = 1:size(beta_hat_l2s,2)
    %     y_val_hat_l2s(:,l) = predict_y(X_val,beta_hat_l2s(:,l));
    %     [acc_l2_all(l),acc_l2_active(l)] = get_accuracy(X_val, y_val_hat_l2s(:,l),y_val,d_sig);
    % end
    % [max_val,max_idx] = max(acc_l2_active);
    % lambda = lambdas(max_idx);

    % Cross validation doesn't really work; it seems lambda=0 always results in
    % the highest accuracy... 

    lambda = 32;

    %% mle

    fprintf('\t MLE: ');
    
    beta_hat_mle = logistic_mle(X_train,y_train);
    y_test_hat_mle = predict_y(X_test,beta_hat_mle);
    [acc_mle_all(n),acc_mle(n)] = get_accuracy(X_test,y_test_hat_mle,y_test,d_sig);

    fprintf('acc_all = %f, acc_active = %f\n',acc_mle_all(n),acc_mle(n));
    
    %% l2 regularization

    fprintf('\t l2: ');
    
    % lambda = add_sigma^2*n_train;
    beta_hat_l2 = logistic_l2(X_train,y_train,lambda);
    y_test_hat_l2 = predict_y(X_test,beta_hat_l2);
    [acc_l2_all(n),acc_l2_active(n)] = get_accuracy(X_test, y_test_hat_l2,y_test,d_sig);

    fprintf('acc_all = %f, acc_active = %f\n',acc_l2_all(n),acc_l2_active(n));
    
    %%
    
    fprintf('\t Dropout, varying delta...\n');
    
    for i = 1:numel(deltas)
        delta = deltas(i);
        add_sigma = add_sigmas(i);
        %% additive noise approximation
% 
%         beta_hat_add_approx = logistic_additive_approx( X_train, y_train, add_sigma );
%         y_test_hat_add_approx = predict_y(X_test,beta_hat_add_approx);
%         [ acc_add_approx_all(n,i), acc_add_approx_active(n,i) ] = get_accuracy( X_test, y_test_hat_add_approx, y_test, d_sig ); 


        %% dropout noise approximation

        fprintf('\t\t delta = %f\n',delta);
        
        epsilon = 1e-3; max_it = 500;
        beta_hat_dropout_approx = logistic_dropout_approx( X_train, y_train, delta, epsilon,max_it);
        y_test_hat_dropout_approx = predict_y(X_test,beta_hat_dropout_approx);
        [acc_dropout_approx_all(n,i), acc_dropout_approx_active(n,i)] ...
          = get_accuracy( X_test, y_test_hat_dropout_approx, y_test, d_sig );
        
        fprintf('\t\t\t approx: acc_all = %f, acc_active = %f\n', ...
                acc_dropout_approx_all(n,i), ...
                acc_dropout_approx_active(n,i));

        %% dropout noise approximation fisher

%         epsilon = 1e-3; max_it = 500;
%         beta_hat_dropout_approx_fisher = logistic_dropout_approx_fisher(X_train, y_train, delta, epsilon,max_it);
%         y_test_hat_dropout_approx_fisher = predict_y(X_test,beta_hat_dropout_approx_fisher);
%         [ acc_dropout_approx_fisher_all(n,i), acc_dropout_approx_fisher_active(n,i) ] = ...
%                 get_accuracy( X_test, y_test_hat_dropout_approx_fisher, y_test, d_sig ); 

%%

  fprintf('\t\t\t Exact, varying num_copies...\n');
        for j = 1:numel(num_copiess)
          
          fprintf('\t\t\t\t num_copies = %d: ',num_copiess(j));
          
          if delta >= 0.9 && num_copiess(j) >= 10
            acc_dropout_all(n,i,j:end) = nan;
            acc_dropout_active(n,i,j:end) = nan;
            fprintf('\n');
            break;
          end
          
            num_copies = num_copiess(j);

            [ X_train_aug, y_train_aug ] = aug_data( X_train, y_train, num_copies, 'dropout', delta );
            [ X_train_augadd, y_train_augadd ] = aug_data( X_train, y_train, num_copies, 'gaussian', add_sigma );

            %% additive noise
%             beta_hat_add = logistic_mle(X_train_augadd,y_train_augadd);
%             y_test_hat_add = predict_y(X_test,beta_hat_add);
%             [acc_add_all(n,i,j),acc_add_active(n,i,j)] = get_accuracy(X_test, y_test_hat_add,y_test,d_sig);

            %% dropout noise
            try
            beta_hat_dropout = logistic_mle(X_train_aug, y_train_aug );
            y_test_hat_dropout = predict_y(X_test,beta_hat_dropout);
            [acc_dropout_all(n,i,j), acc_dropout_active(n,i,j)] = ...
                get_accuracy( X_test, y_test_hat_dropout, y_test, d_sig );
            catch err
              disp(err);
              acc_dropout_all(n,i,j:end) = nan;
              acc_dropout_active(n,i,j:end) = nan;
              break;
            end
            
            fprintf('acc_all = %f, acc_active = %f\n', ...
                    acc_dropout_all(n,i,j),acc_dropout_active(n,i,j));
            
            
        end
    end
    %    waitbar(n/num_trials,wbar);
    fprintf('\n\n');
end
%close(wbar);

%% dropout noise approximation, with unlabeled data

n_unlabeled = 500;
% generating data for unlabeled, so don't actually use/need y_unlabeled
[ X_unlabeled,y_unlabeled] = gen_data(d,d_sig,n_unlabeled,beta);

alphas = 0.1:0.1:1; epsilon = 1e-3;
ms = [10;20;50;100;200;500];
for j=1:length(ms) %floor(size(X_unlabeled,1)/blk_size)
  
  X_tr_idx = 
  
  fprintf('m = %d',ms(j));%j*blk_size);
  for i=1:length(alphas)
    fprintf('\t alpha = %f\n',alphas(i));
    beta_hat_semi_sup = semi_sup_logistic_dropout_approx(X_train,y_train,delta, ...
    %                                                      X_unlabeled(1:j*blk_size,:), ...
                                                         X_unlabeled(1:ms(j),:), ...
                                                         alphas(i),epsilon,max_it);
    y_test_hat_dropout_approx_semi_sup = predict_y(X_test,beta_hat_semi_sup);
    [ semi_sup_acc_dropout_approx_all(i,j), semi_sup_acc_dropout_approx_active(i,j) ] = ...
        get_accuracy(X_test, y_test_hat_dropout_approx_semi_sup, y_test, d_sig );
  end
end


% figure; hold all;
% % max_blk = size(semi_sup_acc_dropout_approx_all,2);
% for i=1:size(semi_sup_acc_dropout_approx_all,1)
%   %  plot(blk_size:blk_size:blk_size*max_blk,semi_sup_acc_dropout_approx_all(i,:));
%   plot(ms,semi_sup_acc_dropout_approx_all(i,:));
% end
% xlabel('Number of unlabeled samples');
% ylabel('Accuracy');
% legend(arrayfun(@(a) ['\alpha = ' num2str(a)],alphas, ...
%                 'UniformOutput',false));
% 
% figure; hold all;
% for j=1:length(ms)
%   plot(alphas,semi_sup_acc_dropout_approx_all(:,j));
% end
% xlabel('Discount factor \alpha');
% ylabel('Accuracy');
% legend(arrayfun(@(m) ['m = ' num2str(m)],blk_size*(1:max_blk), ...
%                 'UniformOutput',false));
% 
% 
% figure;
% surf(blk_size*(1:max_blk),alphas,semi_sup_acc_dropout_approx_all);
% xlabel('Number of unlabeled samples');
% ylabel('Discount factor \alpha');
% zlabel('Accuracy');